Package: ids
Title: Generate Random Identifiers
Version: 1.0.1
Authors@R: person("Rich", "FitzJohn", role = c("aut", "cre"),
    email = "rich.fitzjohn@gmail.com")
Description: Generate random or human readable and pronounceable identifiers.
License: MIT + file LICENSE
URL: https://github.com/richfitz/ids
BugReports: https://github.com/richfitz/ids/issues
Imports: openssl, uuid
Suggests: knitr, rcorpora, rmarkdown, testthat
RoxygenNote: 6.0.1
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2017-05-31 08:43:42 UTC; rich
Author: Rich FitzJohn [aut, cre]
Maintainer: Rich FitzJohn <rich.fitzjohn@gmail.com>
Repository: CRAN
Date/Publication: 2017-05-31 08:49:59 UTC
Built: R 4.6.0; ; 2026-04-26 02:30:21 UTC; unix
