# Generated by roxygen2: do not edit by hand

export(adjective_animal)
export(ids)
export(int_to_proquint)
export(int_to_proquint_word)
export(proquint)
export(proquint_to_int)
export(proquint_word_to_int)
export(random_id)
export(sentence)
export(uuid)
importFrom(uuid,UUIDgenerate)
